@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[143]:
+# In[3]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
     pass
 
 
-# In[144]:
+# In[4]:
 
 
-version = '0.105'
+version = '0.108'
 
 import re
 import tempfile
@@ -33,7 +33,7 @@ import time
 
 # # Helper Functions
 
-# In[145]:
+# In[5]:
 
 
 def is_testing():
@@ -43,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[146]:
+# In[6]:
 
 
 def isnotebook():
@@ -54,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[147]:
+# In[7]:
 
 
 def testPrint(*args):
@@ -64,7 +64,7 @@ def testPrint(*args):
 
 # # Inspect
 
-# In[148]:
+# In[8]:
 
 
 import inspect
@@ -76,7 +76,7 @@ def debugFunctionName():
     return inspect.stack()[1][3]
 
 
-# In[149]:
+# In[9]:
 
 
 debugGetFileName(), debugGetLineNumber(), debugFunctionName()
@@ -84,7 +84,7 @@ debugGetFileName(), debugGetLineNumber(), debugFunctionName()
 
 # # Shell
 
-# In[150]:
+# In[10]:
 
 
 def shell(cmd):
@@ -97,7 +97,7 @@ def shell(cmd):
     return out
 
 
-# In[151]:
+# In[11]:
 
 
 shell_template = """/bin/bash << EOF 2>&1
@@ -119,13 +119,13 @@ def bash(cmd, lineList=True):
 
 # # Timing
 
-# In[152]:
+# In[12]:
 
 
 timeSinceLastCallDict = {}
 
 
-# In[153]:
+# In[13]:
 
 
 def timeSinceLastCall(name):
@@ -138,14 +138,14 @@ def timeSinceLastCall(name):
     return name + ": " + time_s
 
 
-# In[154]:
+# In[14]:
 
 
 if is_testing():
     print(timeSinceLastCall('test marker'))
 
 
-# In[155]:
+# In[15]:
 
 
 import time
@@ -173,7 +173,7 @@ if is_testing():
 
 # # Requiremtns
 
-# In[156]:
+# In[16]:
 
 
 requirements = []
@@ -205,7 +205,7 @@ import yaml
 
 # # Strings
 
-# In[157]:
+# In[17]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -235,7 +235,7 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
-# In[158]:
+# In[18]:
 
 
 def strUuid4():
@@ -246,14 +246,14 @@ def strUuid4():
     return uuid
 
 
-# In[159]:
+# In[19]:
 
 
 if is_testing():
     print(strUuid4())
 
 
-# In[160]:
+# In[20]:
 
 
 def strRand(n, num=True, upperCase=True, lowerCase=True):
@@ -265,7 +265,7 @@ def strRand(n, num=True, upperCase=True, lowerCase=True):
     return ''.join(random.choice(pool) for _ in range(n))
 
 
-# In[161]:
+# In[21]:
 
 
 if is_testing():
@@ -274,7 +274,7 @@ if is_testing():
     print(strRand(20, upperCase=False, lowerCase=False))
 
 
-# In[162]:
+# In[22]:
 
 
 def strMd5Pickle4(obj):
@@ -284,14 +284,14 @@ def strMd5Pickle4(obj):
     return md5.hexdigest()
 
 
-# In[163]:
+# In[23]:
 
 
 if is_testing():
     print(strMd5Pickle4(['a', 'b']))
 
 
-# In[164]:
+# In[24]:
 
 
 def strMd5String(s):
@@ -300,7 +300,7 @@ def strMd5String(s):
     return md5.hexdigest()
 
 
-# In[165]:
+# In[25]:
 
 
 if is_testing():
@@ -309,7 +309,7 @@ if is_testing():
 
 # # Multiprocess / Thread
 
-# In[166]:
+# In[26]:
 
 
 def mapThread(func, args, n=8):
@@ -334,14 +334,14 @@ if is_testing():
 
 # # Lists
 
-# In[167]:
+# In[27]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[168]:
+# In[28]:
 
 
 if is_testing():
@@ -350,7 +350,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[169]:
+# In[29]:
 
 
 def liSplit(a, number_of_splits):
@@ -358,7 +358,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[170]:
+# In[30]:
 
 
 if is_testing():
@@ -370,7 +370,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[171]:
+# In[31]:
 
 
 def liJoin(l):
@@ -380,14 +380,14 @@ def liJoin(l):
     return out
 
 
-# In[172]:
+# In[32]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[173]:
+# In[33]:
 
 
 def liFlatten(l):
@@ -404,7 +404,7 @@ def liFlatten(l):
         return l
 
 
-# In[174]:
+# In[34]:
 
 
 if is_testing():
@@ -414,7 +414,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[175]:
+# In[35]:
 
 
 regex_cache = {}
@@ -424,35 +424,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[176]:
+# In[36]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[177]:
+# In[37]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[178]:
+# In[38]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[179]:
+# In[39]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[180]:
+# In[40]:
 
 
 def lCsv(l, force_oneline=False):
@@ -468,21 +468,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[181]:
+# In[41]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[182]:
+# In[42]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[183]:
+# In[43]:
 
 
 if is_testing():
@@ -491,7 +491,7 @@ if is_testing():
 
 # # Text files
 
-# In[184]:
+# In[44]:
 
 
 def txtread(path):
@@ -512,7 +512,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[185]:
+# In[45]:
 
 
 if is_testing():
@@ -521,21 +521,21 @@ if is_testing():
 
 # # YAML files
 
-# In[186]:
+# In[46]:
 
 
 def yamlwrite(path, obj):
     txtwrite(path, yaml.dump(obj))
 
 
-# In[187]:
+# In[47]:
 
 
 def yamlread(path):
     return yaml.safe_load(txtread(path=path))
 
 
-# In[188]:
+# In[48]:
 
 
 if is_testing():
@@ -548,7 +548,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[189]:
+# In[49]:
 
 
 regex_cache = {}
@@ -560,35 +560,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[190]:
+# In[50]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[191]:
+# In[51]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[192]:
+# In[52]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[193]:
+# In[53]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[194]:
+# In[54]:
 
 
 if is_testing():
@@ -597,7 +597,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[195]:
+# In[55]:
 
 
 import warnings
@@ -621,7 +621,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[196]:
+# In[56]:
 
 
 def gsGsToHttp(path):
@@ -631,7 +631,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[197]:
+# In[57]:
 
 
 def gsLs(path):
@@ -641,7 +641,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[198]:
+# In[58]:
 
 
 def gsExists(path):
@@ -651,7 +651,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[199]:
+# In[59]:
 
 
 def gsUploadFile(gsPath, path):
@@ -659,7 +659,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[200]:
+# In[60]:
 
 
 def gsTxtread(path):
@@ -671,7 +671,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[201]:
+# In[61]:
 
 
 def gsTxtwrite(path, s):
@@ -679,7 +679,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[202]:
+# In[62]:
 
 
 def gsImwrite(path, img):
@@ -688,7 +688,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[203]:
+# In[63]:
 
 
 def gsImread(path):
@@ -697,7 +697,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[204]:
+# In[64]:
 
 
 def gsPklWrite(path, obj):
@@ -705,7 +705,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[205]:
+# In[65]:
 
 
 def gsPklRead(path):
@@ -715,7 +715,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[206]:
+# In[66]:
 
 
 def pklwrite(path, obj):
@@ -728,7 +728,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[207]:
+# In[67]:
 
 
 def pklread(path):
@@ -739,7 +739,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[208]:
+# In[68]:
 
 
 if is_testing():
@@ -752,7 +752,7 @@ if is_testing():
 
 # # Images
 
-# In[209]:
+# In[69]:
 
 
 def _imreadSsh(path):
@@ -819,7 +819,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255, mode='constant'):
         raise RuntimeError(f"Mode '{mode}' not found")
 
 
-# In[210]:
+# In[70]:
 
 
 def imscaleNN(img, s):
@@ -831,7 +831,7 @@ def imscaleBic(img, s):
 
 
 
-# In[211]:
+# In[71]:
 
 
 def imshow(array, scale=1, force_nb=False):
@@ -846,14 +846,14 @@ def imshow(array, scale=1, force_nb=False):
         cv2.waitKey()
 
 
-# In[212]:
+# In[72]:
 
 
 def imNewNoise(height, width, scaleNN=1):
     return imscaleNN((255 * np.random.rand(height, width, 3)).astype(np.uint8), scaleNN)
 
 
-# In[213]:
+# In[73]:
 
 
 if is_testing():
@@ -861,7 +861,7 @@ if is_testing():
     imshow(impad(img, bottom=10, right=10, mode='reflect'))
 
 
-# In[214]:
+# In[74]:
 
 
 def imCropUniformBorder(img, color=255):
@@ -900,7 +900,7 @@ if is_testing():
                         raise RuntimeError('Crop did not work for ', i, j, k, l)
 
 
-# In[215]:
+# In[75]:
 
 
 def imDecodeBytes(imBytes):
@@ -911,7 +911,7 @@ def imDecodeBytes(imBytes):
     return img
 
 
-# In[216]:
+# In[76]:
 
 
 #if is_testing():
@@ -919,14 +919,14 @@ def imDecodeBytes(imBytes):
 #    imshow(img)
 
 
-# In[217]:
+# In[77]:
 
 
 #if is_testing():
 #    imshow('http://via.placeholder.com/70.png')
 
 
-# In[218]:
+# In[78]:
 
 
 if is_testing():
@@ -945,7 +945,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[219]:
+# In[79]:
 
 
 def imRepeat(img, height, width):
@@ -959,7 +959,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[220]:
+# In[80]:
 
 
 if is_testing():
@@ -967,33 +967,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[221]:
+# In[81]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[222]:
+# In[82]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[223]:
+# In[83]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[224]:
+# In[84]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[225]:
+# In[85]:
 
 
 def imAddNoiseGauss(img, std):
@@ -1005,7 +1005,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imNewWhite(150, 150), 30))
 
 
-# In[226]:
+# In[86]:
 
 
 if is_testing():
@@ -1013,7 +1013,7 @@ if is_testing():
     imshow('/tmp/img.jpg')
 
 
-# In[227]:
+# In[87]:
 
 
 def imJpgDegradation(img, quality):
@@ -1026,7 +1026,7 @@ def imJpgDegradation(img, quality):
 #if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[228]:
+# In[88]:
 
 
 def imCropCenter(img, size):
@@ -1041,35 +1041,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[229]:
+# In[89]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[230]:
+# In[90]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[231]:
+# In[91]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[232]:
+# In[92]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[233]:
+# In[93]:
 
 
 def imCropRandom(img, size, auto_size=True):
@@ -1092,7 +1092,7 @@ def imCropRandom(img, size, auto_size=True):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[234]:
+# In[94]:
 
 
 #if is_testing(): 
@@ -1102,7 +1102,7 @@ def imCropRandom(img, size, auto_size=True):
 #    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 41))
 
 
-# In[235]:
+# In[95]:
 
 
 def draw_rect(img, h_beg, h_end, w_beg, w_end, line_width, color=(0, 0, 0)):
@@ -1125,6 +1125,8 @@ def draw_rect(img, h_beg, h_end, w_beg, w_end, line_width, color=(0, 0, 0)):
 
 
 def imCropRandomSeed(img, sizes, idx, crop_idx, use_crop=True, use_minimap=False, mark_color=(200, 50, 50), scale=2, minimap_region=None):
+    if img is None:
+        return imNewWhite(sizes[0], sizes[1])
     
     def get_beg_end(beg_crop, size_crop, size_region, size_img):
         center = beg_crop + (size_crop // 2)
@@ -1164,7 +1166,6 @@ def imCropRandomSeed(img, sizes, idx, crop_idx, use_crop=True, use_minimap=False
         if minimap_region is not None:
             h_new = sizes[0] * minimap_region
             w_new = sizes[1] * minimap_region
-            print(minimap_region)
             h_beg, h_end = get_beg_end(h_start, sizes[0], h_new, h)
             w_beg, w_end = get_beg_end(w_start, sizes[1], w_new, w)
             img = img[h_beg:h_end, w_beg:w_end]
@@ -1183,7 +1184,7 @@ def imCropRandomSeed(img, sizes, idx, crop_idx, use_crop=True, use_minimap=False
         return crop
 
 
-# In[236]:
+# In[96]:
 
 
 if is_testing():
@@ -1191,9 +1192,10 @@ if is_testing():
     imshow(imCropRandomSeed(img, (10, 20), idx=0, crop_idx=0, use_crop=True, use_minimap=True, minimap_region=3), scale=3)
     imshow(imCropRandomSeed(img, (10, 20), idx=0, crop_idx=0, use_crop=False, use_minimap=True, minimap_region=3), scale=3)
     imshow(imCropRandomSeed(img, (10, 20), idx=0, crop_idx=0, use_crop=True, use_minimap=False), scale=3)
+    imshow(imCropRandomSeed(img, (10, 20), idx=0, crop_idx=0, use_crop=True, use_minimap=False), scale=3)
 
 
-# In[237]:
+# In[97]:
 
 
 def imGallery(imgs, pad=0, n_cols=None, n_rows=None):
@@ -1237,7 +1239,7 @@ def imGallery(imgs, pad=0, n_cols=None, n_rows=None):
     return out
 
 
-# In[238]:
+# In[98]:
 
 
 if is_testing():
@@ -1245,7 +1247,7 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[239]:
+# In[99]:
 
 
 if is_testing():
@@ -1259,7 +1261,7 @@ if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1, n_rows=2))
 
 
-# In[240]:
+# In[100]:
 
 
 #if is_testing():
@@ -1267,7 +1269,7 @@ if is_testing():
 #    imshow(imGallery([img40x20, img40x20, img40x20], pad=1))
 
 
-# In[241]:
+# In[101]:
 
 
 def imAddTextHeader(img, text, h=None, size=None):
@@ -1304,7 +1306,7 @@ if is_testing():
 
 # # Video
 
-# In[242]:
+# In[102]:
 
 
 #def imgsToVid(imgs, file_name='out.avi'):
@@ -1324,13 +1326,13 @@ if is_testing():
 #imgsToVid(frames, 'test.avi')
 
 
-# In[243]:
+# In[103]:
 
 
 #ffmpeg -framerate 1/5 -i img%03d.png -c:v libx264 -r 30 -pix_fmt yuv420p out.mp4
 
 
-# In[244]:
+# In[104]:
 
 
 def imsToMp4(path, imgs, time_per_image, frame_rate_output_video=30, verbose=0):
@@ -1350,7 +1352,7 @@ if is_testing():
 
 # # pytorch
 
-# In[245]:
+# In[105]:
 
 
 def ptT(img_np_uint8):
@@ -1367,7 +1369,7 @@ def ptUint8(img_tensor):
     
 
 
-# In[246]:
+# In[106]:
 
 
 if is_testing():
@@ -1379,7 +1381,7 @@ if is_testing():
 
 # # Files
 
-# In[247]:
+# In[107]:
 
 
 def _fiFindByWildcardSsh(wildcard):
@@ -1400,7 +1402,7 @@ def fiFindImgs(dir_path, recursive=False, ext='png'):
     return fiFindByWildcard(wildcard)
 
 
-# In[248]:
+# In[108]:
 
 
 import os
@@ -1423,7 +1425,7 @@ if is_testing():
     print(pexpanduser('~/tmp/file.txt'))
 
 
-# In[249]:
+# In[109]:
 
 
 if is_testing():
@@ -1452,7 +1454,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[250]:
+# In[110]:
 
 
 def fiRemove(path):
@@ -1480,7 +1482,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[251]:
+# In[111]:
 
 
 if is_testing():
@@ -1512,7 +1514,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[252]:
+# In[112]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -1527,7 +1529,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[253]:
+# In[113]:
 
 
 def fiJoin(*args):
@@ -1539,7 +1541,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[254]:
+# In[114]:
 
 
 def jnSettings():
@@ -1550,7 +1552,7 @@ if is_testing():
     jnSettings()
 
 
-# In[255]:
+# In[115]:
 
 
 def jnNbFullWidth():
@@ -1564,7 +1566,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', 100000)
 
 
-# In[256]:
+# In[116]:
 
 
 def jnImageRenderingPixelated():
@@ -1576,7 +1578,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[257]:
+# In[117]:
 
 
 def jnDefault():
@@ -1594,7 +1596,7 @@ if is_testing():
 
 
 
-# In[258]:
+# In[118]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1607,10 +1609,12 @@ if is_testing():
     
 
 
-# In[259]:
+# In[123]:
 
 
 def jnImgToHtmlB64(img):
+    if img is None:
+        return ""
     img = imread(img)
     _, encimg = cv2.imencode('.png', img[:, :, [2, 1, 0]])
     imgB64 = base64.b64encode(encimg).decode('utf-8')
@@ -1619,10 +1623,16 @@ def jnImgToHtmlB64(img):
 
 #def jnHtmlImgsRow(imgs, names=None):
 
-def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, title=None):
+def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, title=None, n_cols=None):
     if isinstance(imgs, OrderedDict):
         names, imgs = list(zip(*imgs.items()))
-        
+    
+    if n_cols is None:
+        n_rows = 1
+        n_cols = len(imgs)
+    else:
+        n_rows = int(np.ceil(len(imgs) / n_cols))
+    
     _row = lambda x: '<tr>' + x + '</tr>'
     _col = lambda x: '<td style="text-align:center; word-wrap:break-word;">' + x + '</td>'
     _nacols = lambda x: prefix_name + "".join(map(_col, x))
@@ -1634,23 +1644,42 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
         _imcols = lambda x: prefix_img + "".join(map(_col, map(lambda y: '<img src="{}" style="display:block; margin:0 auto;">'.format(y), x)))
     else:
         _imcols = lambda x: prefix_img + "".join(map(_col, map(jnImgToHtmlB64, x)))
-        
-    _tab = lambda x: '<table width="{width}%">'.format(width=width) + x + '</table>'
     
-    header = "" if names is None else _row(_nacols(names))
-    html = _tab(header + _row(_imcols(imgs)))
+    _tab = lambda x: '<table width="{width}%">'.format(width=width) + x + '</table>'
+        
+    html = ""
+    
+    
+    def list_select(l, part_idx, n, pad=None):
+        if l is None:
+            return l
+        idx_beg = (row_idx + 0) * n
+        idx_end = (row_idx + 1) * n
+        return [l[idx] if len(l) > idx else pad for idx in range(idx_beg, idx_end)]
+    
+    for row_idx in range(n_rows):
+        
+        cur_names = list_select(names, row_idx, n_cols, pad="")
+        cur_imgs = list_select(imgs, row_idx, n_cols)
+        
+        html_header = "" if cur_names is None else _row(_nacols(cur_names))
+        html_imgs = _row(_imcols(cur_imgs))
+        html += html_header + html_imgs
+    
+    html = _tab(html)
     display(HTML(html))
 
 
-# In[260]:
+# In[124]:
 
 
 if is_testing():
-    jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)], names=["longWordNoSpace"*10, "b", "c"])
+    jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)], names=["longWordNoSpace"*10, "b", "c"], n_cols=None)
+    jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)], names=["longWordNoSpace"*10, "b", "c"], n_cols=2)
    
 
 
-# In[261]:
+# In[125]:
 
 
 if is_testing():
@@ -1660,7 +1689,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[262]:
+# In[126]:
 
 
 
@@ -1668,7 +1697,7 @@ if is_testing():
    jnImageRow(imgs=[imNewBlack(20, 20), imNewBlack(40, 20), imNewBlack(20, 800)])
 
 
-# In[263]:
+# In[127]:
 
 
 #def dfDisplayWithImgs(df):
@@ -1684,7 +1713,7 @@ if is_testing():
 
 # # GIT
 
-# In[264]:
+# In[128]:
 
 
 def gitGetHash():
@@ -1694,7 +1723,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[265]:
+# In[129]:
 
 
 def gitGetBranch():
@@ -1704,7 +1733,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[266]:
+# In[130]:
 
 
 def gitGetMessage():
@@ -1714,7 +1743,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[267]:
+# In[131]:
 
 
 def gitGetAllFiles():
@@ -1724,7 +1753,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[268]:
+# In[132]:
 
 
 def gitListByTime(path=None):
@@ -1740,7 +1769,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[269]:
+# In[133]:
 
 
 def gitTime(path):
@@ -1751,7 +1780,7 @@ if is_testing():
     print(gitTime("__init__.py"))
 
 
-# In[270]:
+# In[134]:
 
 
 def gitTimeRaw(path):
@@ -1762,7 +1791,7 @@ if is_testing():
     print(gitTimeRaw("__init__.py"))
 
 
-# In[271]:
+# In[135]:
 
 
 def shellSortDirsBySize():
@@ -1774,14 +1803,14 @@ if is_testing():
 
 # # Package Files
 
-# In[272]:
+# In[136]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[273]:
+# In[137]:
 
 
 def shell(cmd):
@@ -1789,7 +1818,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[274]:
+# In[138]:
 
 
 if is_testing():
@@ -1821,7 +1850,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[275]:
+# In[139]:
 
 
 if is_testing():
@@ -1831,7 +1860,7 @@ description-file = README.md
 """
 
 
-# In[276]:
+# In[140]:
 
 
 if is_testing():
@@ -1854,14 +1883,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[277]:
+# In[141]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[281]:
+# In[145]:
 
 
 if is_testing():
@@ -1872,7 +1901,7 @@ if is_testing():
     print(output.strip(), 'Version correct: ', output.strip() == version, "(", output.strip(), version, ")")
 
 
-# In[282]:
+# In[146]:
 
 
 if is_testing():
@@ -1883,7 +1912,7 @@ lx/__init__.py
 """
 
 
-# In[283]:
+# In[147]:
 
 
 if is_testing():
